/*
This demo code shows the basic functions of 750W Brushless drive RMCS-6611 for controlling speed and getting feedback from drive. It also demonstrates decelerated stop, brake and enable/disable modbus-analog functions.

Connections of Drive, Arduino and MAX485 TTL to RS485 Converter
Serial Port 0 is not used to connect to drive because its connected to USB-Serial and used to show information on console.

For Arduino Uno Software serial needs to be used as there is only one hardware serial port and its connected to USB-Serial. 
   Drive       MAX485 TTL to RS485 Converter
     A         -       A
     B         -       B
    GND        -      GND
    
   MAX485 TTL to RS485 Converter    Arduino (Software Serial)
     R0                  -            D2              
     DE & RE             -            D8
     DI                  -            D3
     GND                 -            GND
   
For more information see :https://robokits.co.in/motor-drives-drivers/brushless-dc-motor-driver/brushless-motor-driver-dc24-48v-750w-bldc
*/

#include <ModbusMaster.h>   
#include <SoftwareSerial.h> 
SoftwareSerial mySerial(2,3);    //Software Serial port For Arduino Uno
#define MODBUS_DATA_TRANSACTION_PIN 8   //DE & DE Pin of MAX485
ModbusMaster node;                      //object for ModbusMaster

long int a1,a2,a3,a4,a5,a6,a7,a8;
uint8_t Slave_ID = 1;   //Slave ID of Drive

void setup() 
{
    pinMode(MODBUS_DATA_TRANSACTION_PIN, OUTPUT);
    digitalWrite(MODBUS_DATA_TRANSACTION_PIN, 0);
    Serial.begin(9600);     //Baud rate is 9600 8N1
    mySerial.begin(9600);   //Modbus Baud rate is 9600 8N1
    node.begin(mySerial);
    node.preTransmission(preTransmission);
    node.postTransmission(postTransmission);
    Serial.println("RMCS-6611 Brushless DC Motor Driver");
    Serial.print("Slave ID : ");
    Serial.println(Slave_ID);
    
    a1 = node.RS485_Communication(Slave_ID, 1);  // 1-RS485 enable & 2-RS485 disable
    Serial.print("RS485 Communication status : ");
    Serial.println(a1); 
}

void loop() 
{
  Serial.println("Motor is running in Forward Direction");   
  for (int i=0; i<=3000; i+=50)
  {
    a2 = node.Set_Speed(Slave_ID, i);     // Set Speed, speed range is (0-5000 RPM)
    a3 = node.Run_Motor(Slave_ID, 1);     // 1-Forward & 2 Reverse
    GetFeedback();
  }
  
  for(int i=0;i<50;i++){GetFeedback();}  //Wait for 50 cycles, take feedback
  Serial.println("Motor Brake");
  a5 = node.Motor_Brake(Slave_ID, 3);    //Motor Brake command
  for(int i=0;i<50;i++){GetFeedback();}  //Wait for 50 cycles, take feedback
  
  Serial.println("Motor is running in Reverse Direction"); 
  for (int i=3000; i>=0; i-=50)
  {
    a2 = node.Set_Speed(Slave_ID, i);      // Set Speed, speed range is (0-5000 RPM)
    a3 = node.Run_Motor(Slave_ID, 2);      // 1-Forward & 2 Reverse
    GetFeedback();
  }
  
  for(int i=0;i<50;i++){GetFeedback();}   //Wait for 50 cycles, take feedback
  Serial.println("Motor Stop");
  a4 = node.Motor_Stop(Slave_ID, 0);      // Motor Stop command
  for(int i=0;i<50;i++){GetFeedback();}   //Wait for 50 cycles, take feedback
}

void GetFeedback(void) 
{
  a6 = node.Speed_Feedback(Slave_ID);    
  a7 = node.Current_Feedback(Slave_ID);  
  a8 = node.Voltage_Feedback(Slave_ID);
  Serial.print("Set Speed :");
  Serial.print(a2);
  Serial.print("\t\tSpeed_Feedback :");
  Serial.print(a6);  
  Serial.print("\tCurrent_Feedback :");
  Serial.print(a7);
  Serial.print("\tVoltage_Feedback :");
  Serial.println(a8);
}

void preTransmission() 
{
  digitalWrite(MODBUS_DATA_TRANSACTION_PIN, 1);
}

void postTransmission()
{
  digitalWrite(MODBUS_DATA_TRANSACTION_PIN, 0);
}
